/*
 * deagent.h
 * WiFi DataElements Agent private header file.
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * See LICENSE file for license related information.
 *
 */

#ifndef DEAGENT_H
#define DEAGENT_H


#define WIFI_DEVICE_MAX_NUM	4
#define WIFI_IFACE_MAX_NUM	16

struct wifi_netdev {
	char radio[16];
	struct {
		char name[16];
		int mode;
	} iface[WIFI_IFACE_MAX_NUM];
};

enum {
	WIFI_IFACE_FH,
	WIFI_IFACE_BK,
};

#define MAP_PROFILE_1	1
#define MAP_PROFILE_2	2

typedef uint32_t object_t;

#define OBJECT_INVALID 0

#define IEEE1905_OBJECT			"ieee1905"
#define MAP_PLUGIN_OBJECT		"ieee1905.map"
#define WFADATAD_AGENT_OBJECT		"wifi.dataelements"
#define WFADATAD_COLLECTOR_OBJECT	"wifi.dataelements.collector"

struct wifi_assoc_frame {
	uint16_t len;
	uint8_t macaddr[6];
	uint8_t *frame;
	struct list_head list;
};

struct deagent_config {
	uint32_t refresh_int;
	bool is_collector;
	bool is_sap_mode;
	uint32_t profile;
	struct ubus_context *uctx;
	char mapcontroller_conffile[256];
};

#define DELM_OBJECT_PLACEHOLDER			\
	struct ubus_context *ctx;		\
	struct ubus_object obj;			\
	struct ubus_event_handler topology_ev;	\
	struct ubus_event_handler sta_ev;	\
	struct ubus_event_handler ubus_obj_ev;	\
	struct ubus_event_handler wifi_radio_ev;


struct deagent_private {
	struct wifi_netdev ifs[WIFI_DEVICE_MAX_NUM];
	struct deagent_config cfg;
	atimer_t t;
	int getter_running;
	uint8_t cntlr_id[6];		/* controller id */
	bool dm_available;
	struct wifi_data_element *dm;
	struct wifi_data_element *dmi;	/* intermediate */
	struct wifi_sta_events *sv;	/* sta events */
	int pending;
	object_t i1905;			/* i1905 UBUS object */
	object_t i1905_map;		/* i1905.map UBUS object */
	object_t topology;		/* topologyd UBUS object */
	uint8_t ieee1905_macaddr[6];	/* self node macaddr */
	struct cmdu_ackq cmdu_ack_q;
	struct sta_eventlog assoc_stalog;
	//struct ubus_subscriber sub;
	void *subscriber;
	struct list_head framelist;	/* wifi_assoc_frame list */
	struct wifi_radio_events rv;	/* wifi.radio events */
	DELM_OBJECT_PLACEHOLDER
};

#define CMDU_TYPE_INVALID	-1

int deagent_add_device(struct wifi_network_device *remote_data, struct wifi_network_device *data);

extern int deagent_publish_object(struct deagent_private *p, const char *objname);
extern int deagent_register_events(struct deagent_private *p);
extern uint32_t deagent_lookup_object(struct deagent_private *p, const char *objname);


extern int deagent_dump_all(struct deagent_private *p, struct blob_buf *bb);
extern int deagent_dump_all_multiap(struct deagent_private *p, struct blob_buf *bb);
extern int deagent_event_dump(struct deagent_private *p, struct blob_buf *bb);

extern int deagent_get_ieee1905id(struct deagent_private *p);

extern void *deagent_getter_run(void *data);
extern void *deagent_collector_run(void *data);
extern void deagent_free_dm(struct wifi_data_element *dm);

extern char *get_date(time_t *t, char *tbuf);
extern char *get_timestamp(time_t *t, char *tbuf);

// extern void *get_local_device_data(struct deagent_private *p);
// extern int deagent_add_neighbor_node(void *priv, void *remote_dev);
// extern int deagent_del_neighbor_node(void *priv, uint8_t *macaddr);
extern int self_node_topology_query(struct deagent_private *p);
extern void refresh_sta_timer_cb(atimer_t *t);
extern int deagent_get_assoc_sta(struct deagent_private *p, const char *ifname,
		uint8_t *sta_mac, int status_code, char *ev_tsp, uint8_t *bssid);
extern int deagent_get_unassoc_sta(struct deagent_private *p, uint8_t *sta_mac,
		int reason_code, char *ev_tsp);
extern int deagent_fill_sta_bssid(struct deagent_private *p, uint8_t *sta_mac, uint8_t *bssid);
extern int deagent_update_stalog_data(void *sta_log, const char *ifname, uint8_t *mac);
extern int prepare_autoconfig_search(struct deagent_private *priv, uint8_t band);
extern int deagent_subscribe_for_cmdus(struct deagent_private *priv);
extern void get_caps_info(enum wifi_band band,
		struct wifi_caps_element *caps, struct wifi_caps *w_cap,
		uint8_t *bitmap);
extern uint8_t rssi_to_rcpi(int rssi);

#endif /* DEAGENT_H */
