#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libubus.h>
#include <libubox/utils.h>

#include <easy/easy.h>
#include <wifi.h>

#include "timer.h"
#include "debug.h"
#include <cmdu_ackq.h>

#include "eventlog.h"
#include "wfa_delm.h"
#include "deagent.h"


static void free_dev_data(struct wifi_network_device *dev)
{
	int i, j, k, l;
	struct wifi_radio_element *radio = NULL;
	struct wifi_bss_element *bss = NULL;
	struct wifi_opclass_supported_element *sop = NULL;
	struct wifi_scanres_element *scanres = NULL;
	struct wifi_scanres_opclass_element *op = NULL;
	struct wifi_scanres_channel_element *ch = NULL;

	for (i = 0; i < dev->num_radios; i++) {
		radio = dev->radiolist + i;

		for (j = 0; j < radio->num_supp_opclass; j++) {
			sop = radio->supp_opclass + j;

			if (sop->num_exclude_channels > 0)
				free(sop->exclude_chanlist);

			if (sop->num_channels > 0)
				free(sop->chanlist);

			sop->num_exclude_channels = 0;
			sop->num_channels = 0;
		}

		if (radio->num_supp_opclass > 0)
			free(radio->supp_opclass);

		radio->num_supp_opclass = 0;
		radio->supp_opclass = NULL;

		if (radio->num_curr_opclass > 0)
			free(radio->curr_opclass);

		radio->num_curr_opclass = 0;
		radio->curr_opclass = NULL;

		if (radio->num_unassoc_sta > 0)
			free(radio->unassoc_stalist);

		radio->num_unassoc_sta = 0;
		radio->unassoc_stalist = NULL;

		for (j = 0; j < radio->num_bss; j++) {
			bss = radio->bsslist + j;
			if (bss->num_stations > 0)
				free(bss->stalist);

			bss->num_stations = 0;
			bss->stalist = NULL;
		}

		if (radio->num_bss)
			free(radio->bsslist);

		radio->num_bss = 0;
		radio->bsslist = NULL;

		for (j = 0; j < radio->num_scanresult; j++) {
			scanres = radio->scanlist + j;
			if (scanres->num_opclass_scanned > 0) {
				for (k = 0; k < scanres->num_opclass_scanned; k++) {
					op = scanres->opclass_scanlist + k;
					if (op->num_channels_scanned > 0) {
						for (l = 0; l < op->num_channels_scanned; l++) {
							ch = op->channel_scanlist + l;
							if (ch->num_neighbors > 0)
								free(ch->nbrlist);
						}
						free(op->channel_scanlist);
					}
				}
				free(scanres->opclass_scanlist);
			}
		}

		if (radio->num_scanresult > 0)
			free(radio->scanlist);

		radio->num_scanresult = 0;
		radio->scanlist = NULL;
	}

	if (dev->num_radios)
		free(dev->radiolist);

	dev->radiolist = NULL;
	dev->num_radios = 0;

	if (dev->ieee1905_tlv_data)
		free(dev->ieee1905_tlv_data);

	dev->ieee1905_tlv_data = NULL;
}

void deagent_free_dm(struct wifi_data_element *dm)
{
	int i;
	struct wifi_network *net;
	struct wifi_network_device *dev;

	if (dm) {
		fprintf(stderr, "Free old dm\n");
		net = &dm->network;
		if (net->num_devices > 0) {
			for (i = 0; i < net->num_devices; i++) {
				dev = net->device + i;
				free_dev_data(dev);
			}
		}

		free(dm);
	}
}
