/*
 * delm_utils.c
 * WiFi Data Elements utility functions
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * See LICENSE file for license related information.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


char *get_date(time_t *t, char *tbuf)
{
	char tmpbuf[32] = {0};
	const time_t now = time(t);

	if (!tbuf)
		return NULL;

	snprintf(tmpbuf, sizeof(tmpbuf), "%s", ctime(&now));
	tmpbuf[strlen(tmpbuf) - 1] = '\0';
	snprintf(tbuf, 32, "%s", tmpbuf);

	return tbuf;
}

char *get_timestamp(time_t *t, char *tbuf)
{
	char tmpbuf[64] = {0};
	struct tm res;
	char sign;
	long int toff, toff_hour, toff_min;
	const time_t now = time(t);

	if (!tbuf)
		return NULL;

	/* E.g. "2019-02-11T06:42:31.23039-08:00" */

	localtime_r(&now, &res);
	tzset();
	toff = timezone;
	sign = toff > 0 ? '-' : '+';
	toff *= -1L;

	toff_hour = toff / 3600;
	toff_min = (toff % 3600) / 60;

	snprintf(tmpbuf, 63, "%04d-%02d-%02dT%02d:%02d:%02d%c%02ld:%02ld",
					res.tm_year + 1900,
					res.tm_mon + 1,
					res.tm_mday,
					res.tm_hour,
					res.tm_min,
					res.tm_sec,
					sign,
					toff_hour,
					toff_min);

	snprintf(tbuf, 64, "%s", tmpbuf);

	return tbuf;
}

void do_daemonize(const char *pidfile)
{
	char buf[128] = {0};
	int flags;
	int f;

	daemon(0, 0);

	if (!pidfile)
		return;

	f = open(pidfile, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
	if (f) {
		flags = fcntl(f, F_GETFD);
		if (flags != -1) {
			flags |= FD_CLOEXEC;
			fcntl(f, F_SETFD, flags);
		}
		if (lockf(f, F_TLOCK, 0) < 0) {
			fprintf(stderr, "File '%s' exists. Aborting...\n",
				pidfile);
			exit(-1);
		}
		ftruncate(f, 0);
		snprintf(buf, sizeof(buf), "%ld\n", (long)getpid());
		write(f, buf, strlen(buf));
	}
}
